#include <stdio.h>
#include <mpi.h>

// Function to calculate the partial area under the curve X^2 for a given range
double calculate_partial_area(double start, double end, int num_steps, int rank, int size) {
    double width = (end - start) / num_steps;
    double partial_area = 0.0;

    // Loop over the subintervals assigned to the current process
    for (int i = rank; i < num_steps; i += size) {
        double x_start = start + i * width;
        double x_end = x_start + width;
        double height_start = x_start * x_start;
        double height_end = x_end * x_end;

        // Use trapezoidal rule for integration
        partial_area += 0.5 * (height_start + height_end) * width;
    }

    double total_area;

    // Use MPI_Reduce to sum up the partial areas from all processes
    MPI_Reduce(&partial_area, &total_area, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

    return total_area;
}

int main(int argc, char *argv[]) {
    MPI_Init(&argc, &argv);

    int rank, size;
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &size);

    int num_steps = 1000000;  // Adjust based on the required precision

    // Measure execution time
    double start_time = MPI_Wtime();

    // Calculate the total area using MPI parallelization with trapezoidal rule
    double total_area = calculate_partial_area(0.0, 1.0, num_steps, rank, size);

    // Stop measuring time
    double end_time = MPI_Wtime();
    double elapsed_time = end_time - start_time;

    if (rank == 0) {
        // Print the results at the root process
        printf("Total Area (MPI): %f\n", total_area);
        printf("Time (MPI): %f seconds\n", elapsed_time);
    }

    MPI_Finalize();

    return 0;
}
